/** 
 * DevExtreme (ui/pivot_grid/ui.pivot_grid.field_chooser_base.js)
 * Version: 16.1.7
 * Build date: Tue Oct 11 2016
 *
 * Copyright (c) 2012 - 2016 Developer Express Inc. ALL RIGHTS RESERVED
 * EULA: https://www.devexpress.com/Support/EULAs/DevExtreme.xml
 * MAY BE USED WITH: DevExtreme Complete, DevExtreme Web
 */
"use strict";
define(function(require, exports, module) {
    var $ = require("jquery"),
        ArrayStore = require("../../data/array_store"),
        clickEvent = require("../../events/click"),
        commonUtils = require("../../core/utils/common"),
        messageLocalization = require("../../localization/message"),
        registerComponent = require("../../core/component_registrator"),
        Widget = require("../widget/ui.widget"),
        headerFilter = require("../grid_core/ui.grid_core.header_filter"),
        gridCoreUtils = require("../grid_core/ui.grid_core.utils"),
        sorting = require("../grid_core/ui.grid_core.sorting"),
        pivotGridUtils = require("./ui.pivot_grid.utils"),
        Sortable = require("./ui.sortable"),
        inArray = $.inArray,
        each = $.each,
        IE_FIELD_WIDTH_CORRECTION = 1,
        DIV = "<div>",
        HeaderFilterView = headerFilter.HeaderFilterView;
    var processItems = function(groupItems, field) {
        var filterValues = [],
            isTree = !!field.groupName,
            isExcludeFilterType = "exclude" === field.filterType;
        if (field.filterValues) {
            each(field.filterValues, function(_, filterValue) {
                filterValues.push(commonUtils.isArray(filterValue) ? filterValue.join("/") : filterValue)
            })
        }
        pivotGridUtils.foreachTree(groupItems, function(items) {
            var preparedFilterValue, item = items[0],
                path = pivotGridUtils.createPath(items),
                preparedFilterValueByText = isTree ? $.map(items, function(item) {
                    return item.text
                }).reverse().join("/") : item.text;
            item.value = isTree ? path.slice(0) : item.key || item.value;
            preparedFilterValue = isTree ? path.join("/") : item.value;
            if (item.children) {
                item.items = item.children;
                item.children = null
            }
            headerFilter.updateHeaderFilterItemSelectionState(item, item.key && inArray(preparedFilterValueByText, filterValues) > -1 || inArray(preparedFilterValue, filterValues) > -1, isExcludeFilterType)
        })
    };

    function getMainGroupField(dataSource, sourceField) {
        var field = sourceField;
        if (commonUtils.isDefined(sourceField.groupIndex)) {
            field = dataSource.getAreaFields(sourceField.area, true)[sourceField.areaIndex]
        }
        return field
    }
    var FieldChooserBase = Widget.inherit(gridCoreUtils.columnStateMixin).inherit(sorting.sortingMixin).inherit(headerFilter.headerFilterMixin).inherit({
        _getDefaultOptions: function() {
            return $.extend(this.callBase(), {
                allowFieldDragging: true,
                headerFilter: {
                    width: 252,
                    height: 300,
                    texts: {
                        emptyValue: messageLocalization.format("dxDataGrid-headerFilterEmptyValue"),
                        ok: messageLocalization.format("dxDataGrid-headerFilterOK"),
                        cancel: messageLocalization.format("dxDataGrid-headerFilterCancel")
                    }
                }
            })
        },
        _init: function() {
            this.callBase();
            this._headerFilterView = new HeaderFilterView(this);
            this._refreshDataSource();
            this.subscribeToEvents()
        },
        _refreshDataSource: function() {
            var dataSource = this.option("dataSource");
            if (dataSource && dataSource.fields && dataSource.load) {
                this._dataSource = dataSource
            }
        },
        _optionChanged: function(args) {
            switch (args.name) {
                case "dataSource":
                    this._refreshDataSource();
                    break;
                case "headerFilter":
                case "allowFieldDragging":
                    this._invalidate();
                    break;
                default:
                    this.callBase(args)
            }
        },
        renderField: function(field, showColumnLines) {
            var that = this,
                $fieldContent = $(DIV).addClass("dx-area-field-content").text(field.caption || field.dataField),
                $fieldElement = $(DIV).addClass("dx-area-field").addClass("dx-area-box").data("field", field).append($fieldContent),
                mainGroupField = getMainGroupField(that._dataSource, field);
            if ("data" !== field.area) {
                if (field.allowSorting) {
                    that._applyColumnState({
                        name: "sort",
                        rootElement: $fieldElement,
                        column: {
                            alignment: that.option("rtlEnabled") ? "right" : "left",
                            sortOrder: "desc" === field.sortOrder ? "desc" : "asc"
                        },
                        showColumnLines: showColumnLines
                    })
                }
                that._applyColumnState({
                    name: "headerFilter",
                    rootElement: $fieldElement,
                    column: {
                        alignment: that.option("rtlEnabled") ? "right" : "left",
                        filterValues: mainGroupField.filterValues,
                        allowFiltering: mainGroupField.allowFiltering && !field.groupIndex
                    },
                    showColumnLines: showColumnLines
                })
            }
            if (field.groupName) {
                $fieldElement.attr("item-group", field.groupName)
            }
            return $fieldElement
        },
        _clean: function() {},
        _renderContentImpl: function() {
            this._headerFilterView.render(this.element())
        },
        renderSortable: function() {
            var that = this;
            that._createComponent(that.element(), Sortable, $.extend({
                allowDragging: that.option("allowFieldDragging"),
                itemSelector: ".dx-area-field",
                itemContainerSelector: ".dx-area-field-container",
                groupSelector: ".dx-area-fields",
                groupFilter: function() {
                    var dataSource = that._dataSource,
                        $sortable = $(this).closest(".dx-sortable"),
                        pivotGrid = $sortable.data("dxPivotGrid"),
                        pivotGridFieldChooser = $sortable.data("dxPivotGridFieldChooser");
                    if (pivotGrid) {
                        return pivotGrid.getDataSource() === dataSource
                    }
                    if (pivotGridFieldChooser) {
                        return pivotGridFieldChooser.option("dataSource") === dataSource
                    }
                    return false
                },
                itemRender: function($sourceItem, target) {
                    var $item;
                    if ($sourceItem.hasClass("dx-area-box")) {
                        $item = $sourceItem.clone();
                        if ("drag" === target) {
                            $.each($sourceItem, function(index, sourceItem) {
                                $item.eq(index).css("width", parseInt($(sourceItem).css("width"), 10) + IE_FIELD_WIDTH_CORRECTION)
                            })
                        }
                    } else {
                        $item = $(DIV).addClass("dx-area-field").addClass("dx-area-box").text($sourceItem.text())
                    }
                    if ("drag" === target) {
                        var wrapperContainer = $(DIV);
                        $.each($item, function(_, item) {
                            var wrapper = $("<div>").addClass("dx-pivotgrid-fields-container").addClass("dx-widget").append($(item));
                            wrapperContainer.append(wrapper)
                        });
                        return wrapperContainer.children()
                    }
                    return $item
                },
                onDragging: function(e) {
                    var field = e.sourceElement.data("field"),
                        targetGroup = e.targetGroup;
                    e.cancel = false;
                    if (true === field.isMeasure) {
                        if ("column" === targetGroup || "row" === targetGroup || "filter" === targetGroup) {
                            e.cancel = true
                        }
                    } else {
                        if (false === field.isMeasure && "data" === targetGroup) {
                            e.cancel = true
                        }
                    }
                },
                useIndicator: true,
                onChanged: function(e) {
                    var dataSource = that._dataSource,
                        field = e.sourceElement.data("field");
                    e.removeSourceElement = !!e.sourceGroup;
                    that._adjustSortableOnChangedArgs(e);
                    if (field) {
                        dataSource.field(getMainGroupField(dataSource, field).index, {
                            area: e.targetGroup,
                            areaIndex: e.targetIndex
                        });
                        dataSource.load()
                    }
                }
            }, that._getSortableOptions()))
        },
        _adjustSortableOnChangedArgs: function(e) {
            e.removeSourceElement = false;
            e.removeTargetElement = true;
            e.removeSourceClass = false
        },
        _getSortableOptions: function() {
            return {
                direction: "auto"
            }
        },
        subscribeToEvents: function(element) {
            var that = this,
                func = function(e) {
                    var field = $(e.currentTarget).data("field"),
                        mainGroupField = $.extend(true, {}, getMainGroupField(that._dataSource, field)),
                        isHeaderFilter = $(e.target).hasClass("dx-header-filter"),
                        dataSource = that._dataSource;
                    if (isHeaderFilter) {
                        that._headerFilterView.showHeaderFilterMenu($(e.currentTarget), $.extend(mainGroupField, {
                            type: mainGroupField.groupName ? "tree" : "list",
                            dataSource: {
                                load: function(options) {
                                    var userData = options.userData;
                                    if (userData.store) {
                                        return userData.store.load(options)
                                    } else {
                                        var d = $.Deferred();
                                        dataSource.getFieldValues(mainGroupField.index).done(function(data) {
                                            userData.store = new ArrayStore(data);
                                            userData.store.load(options).done(d.resolve).fail(d.reject)
                                        }).fail(d.reject);
                                        return d
                                    }
                                },
                                postProcess: function(data) {
                                    processItems(data, mainGroupField);
                                    return data
                                }
                            },
                            apply: function() {
                                dataSource.field(mainGroupField.index, {
                                    filterValues: this.filterValues,
                                    filterType: this.filterType
                                });
                                dataSource.load()
                            }
                        }))
                    } else {
                        if (field.allowSorting && "data" !== field.area) {
                            dataSource.field(field.index, {
                                sortOrder: "desc" === field.sortOrder ? "asc" : "desc"
                            });
                            dataSource.load()
                        }
                    }
                };
            if (element) {
                element.on(clickEvent.name, ".dx-area-field.dx-area-box", func);
                return
            }
            that.element().on(clickEvent.name, ".dx-area-field.dx-area-box", func)
        },
        _initTemplates: $.noop
    });
    registerComponent("dxPivotGridFieldChooserBase", FieldChooserBase);
    module.exports = FieldChooserBase
});
